import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';

import 'cs_openmediation_model.dart';

typedef InitializationSuccess = void Function();
typedef InitializationFailed = void Function(int code, String message);

typedef RewardedVideoAvailabilityChanged = void Function(bool available);
typedef RewardedVideoAdShowed = void Function(CSOMScene? scene);
typedef RewardedVideoAdShowFailed = void Function(CSOMScene? scene, CSOMError? error);
typedef RewardedVideoAdClicked = void Function(CSOMScene? scene);
typedef RewardedVideoAdClosed = void Function(CSOMScene? scene);
typedef RewardedVideoAdStarted = void Function(CSOMScene? scene);
typedef RewardedVideoAdEnded = void Function(CSOMScene? scene);
typedef RewardedVideoAdRewarded = void Function(CSOMScene? scene);

class CsOpenmediationAd {
  static const MethodChannel _channel = MethodChannel('cs_openmediation_ad');
  static final CsOpenmediationAd _instance = CsOpenmediationAd._internal();
  factory CsOpenmediationAd() => _instance;
  CsOpenmediationAd._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  late InitializationSuccess _initializationSuccess;
  late InitializationFailed _initializationFailed;
  late RewardedVideoAvailabilityChanged _rewardedVideoAvailabilityChanged;
  late RewardedVideoAdShowed _rewardedVideoAdShowed;
  late RewardedVideoAdShowFailed _rewardedVideoAdShowFailed;
  late RewardedVideoAdClicked _rewardedVideoAdClicked;
  late RewardedVideoAdClosed _rewardedVideoAdClosed;
  late RewardedVideoAdStarted _rewardedVideoAdStarted;
  late RewardedVideoAdEnded _rewardedVideoAdEnded;
  late RewardedVideoAdRewarded _rewardedVideoAdRewarded;

  CSOMScene? _getSceneFromArgumeennts(dynamic arguments) {
    if (arguments["id"] != null) {
      int id = arguments["id"];
      String? n = arguments["n"];
      int? isd = arguments["isd"];
      String? oriData = arguments["oriData"];
      return CSOMScene(id, n, isd, oriData);
    }
    return null;
  }

  CSOMError? _getErrorFromArgumeennts(dynamic arguments) {
    if (arguments["code"] != null) {
      int code = arguments["code"];
      String? message = arguments["message"];
      return CSOMError(code, message);
    }
    return null;
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "initializationSuccess":
        _initializationSuccess();
        break;
      case "initializationFailed":
        int code = call.arguments["code"];
        String message = call.arguments["message"];
        _initializationFailed(code, message);
        break;
      case "rewardedVideoAvailabilityChanged":
        bool available = call.arguments["available"];
        _rewardedVideoAvailabilityChanged(available);
        break;
      case "rewardedVideoAdShowed":
        var scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdShowed(scene);
        break;
      case "rewardedVideoAdShowFailed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        CSOMError? error = _getErrorFromArgumeennts(call.arguments);
        _rewardedVideoAdShowFailed(scene, error);
        break;
      case "rewardedVideoAdClicked":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdClicked(scene);
        break;
      case "rewardedVideoAdClosed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdClosed(scene);
        break;
      case "rewardedVideoAdStarted":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdStarted(scene);
        break;
      case "rewardedVideoAdEnded":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdEnded(scene);
        break;
      case "rewardedVideoAdRewarded":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdRewarded(scene);
        break;

      default:
        log('Unknown method ${call.method} ');
    }
  }

  /// 初始话SDK
  Future<void> init(String appKey, bool logEnable) {
    return _channel.invokeMethod("init", <String, dynamic>{
      'appKey': appKey,
      'logEnable': logEnable
    });
  }

  /// 设置用户名
  Future<void> setUserId(String userId) {
    return _channel.invokeMethod("setUserId", <String, dynamic>{
      'userId': userId,
    });
  }

  /// 设置ExtId
  Future<void> setExtId(String sceneName, String extId) {
    return _channel.invokeMethod("setExtId", <String, dynamic>{
      'sceneName': sceneName,
      'extId': extId,
    });
  }

  /// 展示激励视频
  Future<void> showRewardVideo(String sceneName) {
    return _channel.invokeMethod("showRewardVideo", <String, dynamic>{
      'sceneName': sceneName,
    });
  }

  /// 激励视频是否准备好了
  Future<bool?> rewardVideoIsReady() {
    return _channel.invokeMethod<bool>("rewardVideoIsReady");
  }

  Future<void> onResume() {
    return _channel.invokeMethod("onResume");
  }

  Future<void> onPause() {
    return _channel.invokeMethod("onPause");
  }

  /// SDK初始化成功
  void setInitializationSuccessListener(InitializationSuccess initializationSuccess) => _initializationSuccess = initializationSuccess;

  /// SDK初始化失败
  void setInitilizationFailedListener(InitializationFailed initializationFailed) => _initializationFailed = initializationFailed;

  void setRewardedVideoAvailabilityChangedListener(RewardedVideoAvailabilityChanged rewardedVideoAvailabilityChanged) => _rewardedVideoAvailabilityChanged = rewardedVideoAvailabilityChanged;
  void setRewardedVideoAdShowedListener(RewardedVideoAdShowed rewardedVideoAdShowed) => _rewardedVideoAdShowed = rewardedVideoAdShowed;
  void setRewardedVideoAdShowFailedListener(RewardedVideoAdShowFailed rewardedVideoAdShowFailed) => _rewardedVideoAdShowFailed = rewardedVideoAdShowFailed;
  void setRewardedVideoAdClickedListener(RewardedVideoAdClicked rewardedVideoAdClicked) => _rewardedVideoAdClicked = rewardedVideoAdClicked;
  void setRewardedVideoAdClosedListener(RewardedVideoAdClosed rewardedVideoAdClosed) => _rewardedVideoAdClosed = rewardedVideoAdClosed;
  void setRewardedVideoAdStartedListener(RewardedVideoAdStarted rewardedVideoAdStarted) => _rewardedVideoAdStarted = rewardedVideoAdStarted;
  void setRewardedVideoAdEndedListener(RewardedVideoAdEnded rewardedVideoAdEnded) => _rewardedVideoAdEnded = rewardedVideoAdEnded;
  void setRewardedVideoAdRewardedListener(RewardedVideoAdRewarded rewardedVideoAdRewarded) => _rewardedVideoAdRewarded = rewardedVideoAdRewarded;
}
